# -*- coding: utf-8 -*-
##############################################################################
#
#    school module for OpenERP
#    Copyright (C) 2011 Zikzakmedia S.L. (http://www.zikzakmedia.com)
#      Jesús Martín Jiménez <jmargin@zikzakmedia.com> All Rights Reserved.
#
#    This file is a part of school module
#
#    school OpenERP module is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    school OpenERP module is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program. If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv, fields
from tools.translate import _

import time
import mx.DateTime


class holiday_year_wizard(osv.osv_memory):
    _name = 'school.holidays_calendar.wizard'

    _columns = {
        'year' : fields.integer('Year', required=True),
    }

    _defaults = {
        'year' : lambda *a: int(time.strftime('%Y')),
    }

    def _check_date(self, cr, uid, ids, context=None):
        return self.browse(cr, uid, ids[0], context=context).year >= int(time.strftime('%Y'))

    def action_cancel(self, cr, uid, ids, context=None):
        return {'type': 'ir.actions.act_window_close'}

    def action_apply(self, cr, uid, ids, context=None):
        if context is None:
            context={}
        
        this = self.browse(cr, uid, ids[0], context=context)

        first_day = mx.DateTime.strptime('%04s-01-01' % (this.year,), '%Y-%m-%d')
        last_day = mx.DateTime.strptime('%04s-12-31' % (this.year,), '%Y-%m-%d')

        tmp_day = first_day

#        holiday_calendar = self.pool.get('school.holidays_calendar')
#        year_id = holiday_calendar.create(cr, uid, {'name' : str(this.year),}, context=context)

        holiday_interval = self.pool.get('school.holiday_interval')
        holiday_ids = []

        counter = 1
        while tmp_day <= last_day:
            if tmp_day.day_of_week == mx.DateTime.Saturday:
                saturday = tmp_day.strftime('%Y-%m-%d')
                tmp_day = tmp_day + mx.DateTime.RelativeDate(days=1)
                sunday = tmp_day.strftime('%Y-%m-%d')
                holiday_ids.append(holiday_interval.create(cr, uid, {
                    'date_from' : saturday,
                    'date_to' : sunday,
                    'name' : _('Weekend %02d of %04s') % (counter, str(this.year)),
#                    'calendars' :  [(6,0,[year_id])],
                }, context=context))
                counter = counter + 1
            tmp_day = tmp_day + mx.DateTime.RelativeDate(days=1)
        return {
            'view_type': 'form',
            "view_mode": 'tree,form',
            'res_model': 'school.holiday_interval',
            'view_id':self.pool.get('ir.ui.view').search(cr,uid,[('name','=','school.holiday_interval.tree.all')]),
            'type': 'ir.actions.act_window',
            'target': 'current',
            'res_id' : holiday_ids,
        }

    _constraints = [(_check_date, "Can you check the year", ['year']),]

holiday_year_wizard()
