# -*- encoding: utf-8 -*-
##############################################################################
#
#    school module for OpenERP
#    Copyright (C) 2010 Tecnoba S.L. (http://www.tecnoba.com)
#       Pere Ramon Erro Mas <pereerro@tecnoba.com> All Rights Reserved.
#
#    This file is a part of school module
#
#    school OpenERP module is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    school OpenERP module is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv, fields
from lxml import etree
import zipfile
from StringIO import StringIO
import base64
from datetime import datetime
from tools.translate import _

week_days=[('1','monday'),('2','tuesday'),('3','wednesday'),('4','thursday'),('5','friday'),('6','saturday'),('7','sunday')]

class school_import_class_group_static(osv.osv):
    _name = 'school.import_nottelmann_class_group_static'
    _columns = {
        'code': fields.char('Code',size=20,),
        'group_id' : fields.many2one('groups.group',string='Internal group',ondelete='cascade',),
    }
    _sql_constraints = [('code_unique','UNIQUE (code)','The code must be unique'),]
school_import_class_group_static()
    
class school_import_class_group(osv.osv_memory):
    _name = 'school.import_nottelmann_class_group'
    _columns = {
        'name' : fields.char('Name',size=200,),
        'import_id' : fields.many2one('school.import_nottelmann_data',string='Import data',ondelete='cascade',),
        'group_id' : fields.many2one('groups.group',string='Internal group',ondelete='cascade',),
        'code': fields.char('Code',size=20,),
    }
school_import_class_group()

class school_import_data(osv.osv_memory):
    _name = 'school.import_nottelmann_data'
    _columns = {
        'date_creation' : fields.datetime('Date creation',),
        'file_zip' : fields.binary('Zipped XML', filters='*.zip', help='Zipped XML to import',),
        'class_group' : fields.one2many('school.import_nottelmann_class_group','import_id','Groups',),
        'nclasses_at_time' : fields.integer('Num Classes at time',readonly=True,),
        'n_rooms_d' : fields.integer('N rooms readed',readonly=True,),
        'n_rooms_i' : fields.integer('N rooms imported',readonly=True,),
        'n_subjects_d' : fields.integer('N subjects readed',readonly=True,),
        'n_subjects_i' : fields.integer('N subjects imported',readonly=True,),
        'n_teachers_d' : fields.integer('N teachers readed',readonly=True,),
        'n_teachers_i' : fields.integer('N teachers imported',readonly=True,),
        'n_classes_d' : fields.integer('N classes readed',readonly=True,),
        'n_classes_i' : fields.integer('N classes imported',readonly=True,),
        'total_classes' : fields.integer('Total classes',readonly=True,),
        'last_classes' : fields.integer('last_classes',readonly=True),
        'percent' : fields.float('Completed',readonly=True,),
        'minutes_left' : fields.integer('Minutes remaining',readonly=True,),
    }

    _defaults = {
        'last_classes' : lambda *a: 1,
        'nclasses_at_time' : lambda *a: 20,
        'date_creation' : lambda *a: datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
    }

    def add_mark(model,xml_id):
        if model=='res.partner': xml_id='rp-'+xml_id
        if model=='school.teacher': xml_id='t-'+xml_id
        if model=='res.partner.contact': xml_id='rpc-'+xml_id
        return xml_id


    def get_id_from_xmlid(self, cr, uid, model, xml_id, context=None):
        imd_obj=self.pool.get('ir.model.data')
        id=False
        ir_model_data_ids=imd_obj.search(cr, uid, [('model','=',model),('name','=',xml_id)])
        if ir_model_data_ids:
            for data in imd_obj.read(cr, uid, ir_model_data_ids, ['res_id']):
                if not self.pool.get(model).search(cr, uid, [('id','=',data['res_id'])]):
                    imd_obj.unlink(cr, uid, [data['id']])
                else:
                    id=data['res_id']
        return id

    def preload(self, cr, uid, ids, context=None):
        obj=self.pool.get('school.import_nottelmann_class_group_static')
        for imp in self.browse(cr, uid, ids):
            val =base64.decodestring(imp.file_zip)
            fp = StringIO()
            fp.write(val)
            fdata = zipfile.ZipFile(fp, 'r')
            fname = fdata.namelist()[0]
            root = etree.fromstring(fdata.read(fname))
            if root.tag=='document' and root.get('version')=="1.1":
                class_nodes=root.findall('classes/class')
                if class_nodes!=None:
                    for class_node in class_nodes:
                        group_ids=[]
                        ids2=obj.search(cr, uid, [('code','=',class_node.get('id')),])
                        if ids2: group_ids=[x.group_id.id for x in obj.browse(cr, uid, ids2) if x.group_id]
                        longname_node=class_node.find('longname')
                        self.pool.get('school.import_nottelmann_class_group').create(cr, uid, {'import_id': imp.id, 'group_id': group_ids and group_ids[0] or False, 'name': longname_node!=None and longname_node.text or class_node.get('id'), 'code': class_node.get('id'), })

        return {
                'name': _('Import GPUntis Timetable Data: Link groups'),
                'view_type': 'form',
                'view_mode': 'form',
                'res_model': 'school.import_nottelmann_data',
                'res_id' : ids[0],
                'views' : [(self.get_id_from_xmlid(cr, uid, 'ir.ui.view', 'school_nottelmann_import_form_view_1', context=context),'form')],
                'view_id': False,
                'target' : 'new',
                'type': 'ir.actions.act_window',
        }


    def planifica(self, cr, uid, ids, context=None):
        for imp in self.browse(cr, uid, ids):
            cron_id=self.pool.get('ir.cron').create(cr, uid, {'name': 'importacio', 'active': False, 'numbercall': 0, 'model': 'school.import_nottelmann_data', 'function': 'load', },)
            self.pool.get('ir.cron').write(cr, uid, [cron_id], {'active': True, 'numbercall': -1, 'interval_number': 1, 'interval_type': 'minutes', 'model': 'school.import_nottelmann_data', 'function': 'load', 'args': ([imp.id],cron_id,imp.date_creation,context,),},)
        return {
                'name': _('Import GPUntis Timetable Data: End Wizard'),
                'view_type': 'form',
                'view_mode': 'form',
                'res_model': 'school.import_nottelmann_data',
                'res_id' : ids[0],
                'views' : [(self.get_id_from_xmlid(cr, uid, 'ir.ui.view', 'school_nottelmann_import_form_view_2', context=context),'form')],
                'view_id': False,
                'type': 'ir.actions.act_window',
                'auto_refresh' : 1,
        }

    def load(self, cr, uid, ids, cron_id, date_creation, context=None):
        obj=self.pool.get('school.import_nottelmann_class_group_static')
        imd_obj=self.pool.get('ir.model.data')
        limit=0
        if len(ids)!=len(self.search(cr, uid, [('id','in',ids),('date_creation','=',date_creation)])):
            self.pool.get('ir.cron').unlink(cr, uid, [cron_id])
            return
        for imp in self.browse(cr, uid, ids):
            for item in imp.class_group:
                obj.unlink(cr,uid,obj.search(cr,uid,[('code','=',item.code)]))
                if item.group_id:
                    obj.create(cr,uid,{'code': item.code, 'group_id': item.group_id.id,})
            vals2={}
            val =base64.decodestring(imp.file_zip)
            fp = StringIO()
            fp.write(val)
            fdata = zipfile.ZipFile(fp, 'r')
            fname = fdata.namelist()[0]
            root = etree.fromstring(fdata.read(fname))

            times={}
            rooms={}
            subjects={}
            teachers={}
            classes={}
            if root.tag=='document' and root.get('version')=="1.1":
                begindate=datetime.strptime(root.find('general/termbegindate').text,'%Y%m%d')
                enddate=datetime.strptime(root.find('general/termenddate').text,'%Y%m%d')

                time_periods=root.findall('timeperiods/timeperiod')
                if time_periods!=None:
                    for time_period in time_periods:
                        tid=(int(time_period.find('day').text),int(time_period.find('period').text),)
                        times[tid]={}
                        times[tid]['week_day']=reduce( lambda x,y: time_period.find('day').text==y[0] and y or x, [(False,False)]+week_days )[0]
                        hour_from=time_period.find('starttime').text
                        times[tid]['hour_from']=float(hour_from[0:2])+float(hour_from[2:4])/60
                        hour_to=time_period.find('endtime').text
                        times[tid]['hour_to']=float(hour_to[0:2])+float(hour_to[2:4])/60
                vals2['n_rooms_d']=0
                vals2['n_rooms_i']=imp.n_rooms_i
                room_nodes=root.findall('rooms/room')
                if room_nodes!=None:
                    for room_node in room_nodes:
                        xmlid=room_node.get('id')
                        vals2['n_rooms_d']+=1
                        room_id=self.get_id_from_xmlid(cr, uid, 'school.room', self.add_mark('school.room')+xmlid)
                        if not room_id:
                            room_id=self.pool.get('school.room').create(cr, uid, {'name': room_node.find('longname').text, })
                            vals2['n_rooms_i']+=1
                            imd_obj.create(cr, uid, {'model': 'school.room', 'name': self.add_mark('school.room')+xmlid, 'res_id': room_id})
                        rooms[xmlid]=room_id
                        
                vals2['n_subjects_d']=0
                vals2['n_subjects_i']=imp.n_subjects_i
                subject_nodes=root.findall('subjects/subject')
                if subject_nodes!=None:
                    for subject_node in subject_nodes:
                        xmlid=subject_node.get('id')
                        vals2['n_subjects_d']+=1
                        course_id=self.get_id_from_xmlid(cr, uid, 'school.course', self.add_mark('school.course')+xmlid)
                        if not course_id:
                            longname_node=subject_node.find('longname')
                            course_id=self.pool.get('school.course').create(cr, uid, {'name': longname_node!=None and longname_node.text or xmlid, 'code': xmlid, })
                            vals2['n_subjects_i']+=1
                            imd_obj.create(cr, uid, {'model': 'school.course', 'name': self.add_mark('school.course')+xmlid, 'res_id': course_id})
                        subjects[xmlid]=course_id

                vals2['n_teachers_d']=0
                vals2['n_teachers_i']=imp.n_teachers_i
                teacher_nodes=root.findall('teachers/teacher')
                if teacher_nodes!=None:
                    for teacher_node in teacher_nodes:
                        xmlid=teacher_node.get('id')
                        vals2['n_teachers_d']+=1
                        teacher_id=self.get_id_from_xmlid(cr, uid, 'school.teacher', self.add_mark('school.teacher')+xmlid)
                        if not teacher_id:
                            teacher_id=self.pool.get('school.teacher').create(cr, uid, {'name': teacher_node.find('surname').text, })
                            vals2['n_teachers_i']+=1
                            imd_obj.create(cr, uid, {'model': 'school.teacher', 'name': self.add_mark('school.teacher')+xmlid, 'res_id': teacher_id})
                        teachers[xmlid]=teacher_id

                class_nodes=root.findall('classes/class')
                if class_nodes!=None:
                    for class_node in class_nodes:
                        xmlid=class_node.get('id')
                        for item in imp.class_group:
                            if item.code==xmlid and item.group_id: classes[xmlid]=item.group_id.id

                vals2['n_classes_d']=0
                vals2['n_classes_i']=imp.n_classes_i
                lesson_nodes=root.findall('lessons/lesson')
                if lesson_nodes!=None:
                    vals2['total_classes']=len(lesson_nodes)
                    for lesson_node in lesson_nodes:
                        xmlid=lesson_node.get('id')
                        vals2['n_classes_d']+=1
                        classe_id=self.get_id_from_xmlid(cr, uid, 'school.classe', self.add_mark('school.classe')+xmlid)
                        if not classe_id:
                            if limit>imp.nclasses_at_time: break
                            vals={'date_from': begindate.strftime('%Y-%m-%d'), 'date_to': enddate.strftime('%Y-%m-%d')}
                            subject_node=lesson_node.find('lesson_subject')
                            subject_xmlid=None
                            if subject_node!=None:
                                subject_xmlid=subject_node.get('id')
                            if not subject_xmlid:
                                resids=self.pool.get('school.course').search(cr, uid, [('code','=','--')])
                                if resids:
                                    vals['course_id']=resids[0]
                                else:
                                    vals['course_id']=self.pool.get('school.course').create(cr, uid, {'name': '--', 'code':'--',},)
                            else:
                                vals['course_id']=subjects[subject_xmlid]
                            classes_node=lesson_node.find('lesson_classes')
                            if classes_node!=None:
                                classe_xmlid=classes_node.get('id')
                                if classe_xmlid not in classes:
                                    separator=' '
                                    classe_xmlids=classe_xmlid.split(separator)
                                    # omplim la llista dels identificadors de classe list_xmlids
                                    list_xmlids=[]
                                    while classe_xmlids:
                                        g=0
                                        while g<len(classe_xmlids):
                                            classe_xmlid2=reduce(lambda x,y: x+separator+y,map(lambda x: classe_xmlids[x],range(g+1)))
                                            if classe_xmlid2 in classes:
                                                list_xmlids.append(classe_xmlid2)
                                                classe_xmlids=classe_xmlids[g+1:]
                                                g=0
                                            else: g+=1
                                        classe_xmlids=classe_xmlids[1:]
                                    classe_xmlids=list_xmlids
                                    if classe_xmlids>1:
                                        group_ids=map(lambda x: classes[x],classe_xmlids)
                                        query="SELECT father FROM groups_parent_groups_rel WHERE son IN %s GROUP BY father HAVING count(*)=%s LIMIT 1"
                                        cr.execute(query,(tuple(group_ids),len(group_ids)))
                                        father=[x for (x,) in cr.fetchall()]
                                        if not father:
                                            classes[classe_xmlid]=self.pool.get('groups.group').create(cr, uid, {'name': classe_xmlid, 'children_ids': [(6,0,group_ids)]})
                                        else: classes[classe_xmlid]=father[0]
                                vals['group_id']=classes[classe_xmlid]
                            classe_id=self.pool.get('school.classe').create(cr, uid, vals)
                            vals2['n_classes_i']+=1
                            limit+=1
                            imd_obj.create(cr, uid, {'model': 'school.classe', 'name': self.add_mark('school.classe')+xmlid, 'res_id': classe_id})
                            for time_node in lesson_node.findall('times/time'):
                                tkey=(int(time_node.find('assigned_day').text),int(time_node.find('assigned_period').text))
                                vals=dict(times[tkey])
                                room_node=time_node.find('assigned_room')
                                if room_node!=None:
                                    room_xmlid=room_node.get('id')
                                    if room_xmlid and room_xmlid in rooms:
                                        vals['room_id']=rooms[room_xmlid]
                                vals['classe_id']=classe_id
                                iwl_id=self.pool.get('school.impartition_week_line').create(cr, uid, vals)
                                teacher_node=lesson_node.find('lesson_teacher')
                                if teacher_node!=None and teacher_node.get('id') in teachers:
                                    self.pool.get('school.teacher_data').create(cr, uid, {'iwl_id': iwl_id, 'teacher_id': teachers[teacher_node.get('id')]})
                                cr.connection.commit()
            if limit==0:
                self.pool.get('ir.cron').unlink(cr, uid, [cron_id])
            vals2['last_classes']=limit
            if vals2['total_classes']:
                vals2['percent']=vals2['n_classes_d']*100/vals2['total_classes']
                vals2['minutes_left']=int((vals2['total_classes']-vals2['n_classes_d'])/imp.nclasses_at_time)
            else:
                vals2['percent']=100
                vals2['minutes_left']=0
            if vals2: self.write(cr, uid, [imp.id], vals2)




    
school_import_data()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

