# -*- encoding: utf-8 -*-
##############################################################################
#
#    school module for OpenERP
#    Copyright (C) 2010 Tecnoba S.L. (http://www.tecnoba.com)
#       Pere Ramon Erro Mas <pereerro@tecnoba.com> All Rights Reserved.
#
#    This file is a part of school module
#
#    school OpenERP module is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    school OpenERP module is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv, fields
from lxml import etree
import zipfile
from StringIO import StringIO
import base64
from datetime import datetime
from tools.translate import _

class school_saga_tutor_to_import(osv.osv_memory):
    _name = 'school.saga_tutor_to_import'
    _max_count = 20000

    def _contact_tutor(self, cr, uid, ids, field_name, args, context=None):
        ret={}
        for tutor in self.browse(cr, uid, ids):
            ret[tutor.id]=[]
            cpf_obj=self.pool.get('contact.partner.function')
            cpf_ids=cpf_obj.search(cr, uid, [('partner_id','=',tutor.tutor_id.id),('function_id','=',tutor.importat.function_id.id)])
            for data in cpf_obj.read(cr, uid, cpf_ids, ['contact_id']):
                ret[tutor.id].append(type(data['contact_id'])!=type(0) and data['contact_id'][0] or data['contact_id'])

        return ret

    def _tutors_not_linked(self, cr, uid, ids, field_name, args, context=None):
        query="SELECT rp.id FROM res_partner AS rp LEFT JOIN ir_model_data AS imd ON rp.id=imd.res_id AND imd.model='res.partner' WHERE imd.id IS NULL"
        cr.execute(query)
        return {}.fromkeys(ids,",".join(['0']+[str(x) for (x,) in cr.fetchall()]))

    _columns = {
        'tutor_id' : fields.many2one('res.partner','Tutor', ),
        'tutors_not_linked' : fields.function(_tutors_not_linked,method=True,type='text',),
        'xml_id' : fields.char('Id SAGA',size=250,),
        'address_id' : fields.many2one('res.partner.address','Address',),
        'tutor_id_by_xmlid' : fields.many2one('res.partner','Tutor linked', ),
        'contact_ids' : fields.function(_contact_tutor,method=True,type='many2many',obj='res.partner.contact',string='Students',),
        'importat' : fields.many2one('school.import_saga_data','Import',),
        'comment' : fields.text('Comment'),
        'name' : fields.char('Name', size=255,),
        'name_in' : fields.related('tutor_id','name',type='char',string='Name In',size=200,),
        'change_name' : fields.boolean('Change Name',),
        'vat': fields.char('VAT',size=15,),
        'vat_in' : fields.related('tutor_id','vat',type='char',string='Vat In',size=200,),
        'change_vat' : fields.boolean('Change vat',),
        'doc_type': fields.char('Doc type',size=50,),
        'doc_type_in' : fields.related('tutor_id','doc_type',type='char',string='Doc type In',),
        'change_doc_type' : fields.boolean('Change Doc type',),
        'street' : fields.char('Street', size=250,),
        'street_in' : fields.related('address_id','street',type='char',string='Street In',size=200,),
        'change_street' : fields.boolean('Change street',),
        'city' : fields.char('City', size=250,),
        'city_in' : fields.related('address_id','city',type='char',string='City In',size=200,),
        'change_city' : fields.boolean('Change city'),
        'zip' : fields.char('ZIP', size=20,),
        'zip_in' : fields.related('address_id','zip',type='char',string='ZIP In',),
        'change_zip' : fields.boolean('Change ZIP',),
        'changed' : fields.boolean('Changed'),
    }

    def dona_valors_a_changes(self, cr, uid, ids, context=None):
        fields_changing=['change_name','change_vat','change_doc_type','change_street','change_city','change_zip']
        for item in self.browse(cr, uid, ids, context=context):
            vals=dict.fromkeys(['address_id']+fields_changing,False)
            if item.tutor_id:
                vals['change_name']=(item.tutor_id.name!=item.name)
                vals['change_vat']=(item.tutor_id.vat!=item.vat)
                vals['change_doc_type']=(item.tutor_id.doc_type!=item.doc_type)
                for address in item.tutor_id.address:
                    vals['address_id']=address.id
                    vals['change_street']=(address.street!=item.street)
                    vals['change_city']=(address.city!=item.city)
                    vals['change_zip']=(address.zip!=item.zip)
            vals['changed']=False
            for x in fields_changing:
                vals['changed']=(vals['changed'] or vals[x])
                if vals['changed']: break
            self.write(cr, uid, [item.id], vals)
    
    def create(self, cr, uid, vals, context=None):
        ret=super(school_saga_tutor_to_import, self).create(cr, uid, vals, context=context)
        if ret: self.dona_valors_a_changes(cr, uid, [ret], context)
        return ret

    def write(self, cr, uid, ids, vals, context=None):
        ret=super(school_saga_tutor_to_import, self).write(cr, uid, ids, vals, context=context)
        if ret and 'tutor_id' in vals: self.dona_valors_a_changes(cr, uid, ids, context)
        return ret

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        if not context: context={}
        ret=super(school_saga_tutor_to_import, self).search(cr, uid, args, offset=offset, limit=limit, order=order, context=context, count=0)
        for (field,op,value) in args:
            for (id,data) in self.datas.items():
                if op=='=' and data[field]!=value and id in ret: ret.remove(id)
        # The order is important only in Desktop!!  (?)
        if context.get('only_data_changed',False):
             for (id,data) in self.datas.items():
                if not data['changed'] and id in ret: ret.remove(id)
        elif context.get('only_not_linked',False):
             for (id,data) in self.datas.items():
                if data['tutor_id_by_xmlid'] and id in ret: ret.remove(id)
        if count: return len(ret)
        return ret

school_saga_tutor_to_import()

class school_saga_student_to_import(osv.osv_memory):
    _name = 'school.saga_student_to_import'
    _max_count = 20000

    def _contacts_not_linked(self, cr, uid, ids, field_name, args, context=None):
        query="SELECT rpc.id FROM res_partner_contact AS rpc LEFT JOIN ir_model_data AS imd ON rpc.id=imd.res_id AND imd.model='res.partner.contact' WHERE imd.id IS NULL"
        cr.execute(query)
        return {}.fromkeys(ids,",".join(['0']+[str(x) for (x,) in cr.fetchall()]))

    _columns = {
        'contact_id' : fields.many2one('res.partner.contact','Student',),
        'contacts_not_linked' : fields.function(_contacts_not_linked,method=True,type='text',),
        'contact_id_by_xmlid' : fields.many2one('res.partner.contact','Student linked',),
        'importat' : fields.many2one('school.import_saga_data','Import',),
        'xml_id' : fields.char('Id SAGA',size=250,),
        'comment' : fields.text('Comment'),
        'name' : fields.char('Name',size=259,),
        'name_in' : fields.related('contact_id','name',string='Name In',type='char',size=200,),
        'change_name' : fields.boolean('Change name',),
        'first_name' : fields.char('First name',size=259,),
        'first_name_in' : fields.related('contact_id','first_name',string='First name In',type='char',size=200,),
        'change_first_name' : fields.boolean('Change first name',),
        'birthdate' : fields.date('Birthdate',),
        'birthdate_in' : fields.related('contact_id','birthdate',type='date',string='Birthdate In',),
        'change_birthdate' : fields.boolean('Change birthdate',),
        'vat': fields.char('VAT',size=15,),
        'vat_in' : fields.related('contact_id','vat',type='char',string='VAT In',size=200,),
        'change_vat' : fields.boolean('Change VAT',),
        'doc_type': fields.char('Doc type',size=50,),
        'doc_type_in' : fields.related('contact_id','doc_type',type='char',string='Doc type In',size=200,),
        'change_doc_type' : fields.boolean('Change Doc type',),
        'gender' : fields.char('Gender',size=15,),
        'gender_in' : fields.related('contact_id','gender',type='char',string='Gender In',size=200,),
        'change_gender' : fields.boolean('Change Gender',),
        'changed' : fields.boolean('Changed'),
        'xml_id_tutor1' : fields.char('XML tutor 1',size=200,),
        'xml_id_tutor2' : fields.char('XML tutor 2',size=200,),
    }

    def dona_valors_a_changes(self, cr, uid, ids, context=None):
        fields_changing=['change_name','change_first_name','change_birthdate','change_vat','change_doc_type','change_gender',]
        for item in self.browse(cr, uid, ids, context=context):
            vals=dict.fromkeys(fields_changing,False)
            if item.contact_id:
                vals['change_name']=(item.contact_id.name!=item.name)
                vals['change_first_name']=(item.contact_id.first_name!=item.first_name)
                vals['change_birthdate']=(item.contact_id.birthdate!=item.birthdate)
                vals['change_vat']=(item.contact_id.vat!=item.vat)
                vals['change_doc_type']=(item.contact_id.doc_type!=item.doc_type)
                vals['change_gender']=(item.contact_id.gender!=item.gender)
            vals['changed']=False
            for x in fields_changing:
                vals['changed']=(vals['changed'] or vals[x])
                if vals['changed']: break
            self.write(cr, uid, [item.id], vals)

    def create(self, cr, uid, vals, context=None):
        ret=super(school_saga_student_to_import, self).create(cr, uid, vals, context=context)
        if ret: self.dona_valors_a_changes(cr, uid, [ret], context)
        return ret

    def write(self, cr, uid, ids, vals, context=None):
        ret=super(school_saga_student_to_import, self).write(cr, uid, ids, vals, context=context)
        if ret and 'contact_id' in vals: self.dona_valors_a_changes(cr, uid, ids, context)
        return ret

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        if not context: context={}
        ret=super(school_saga_student_to_import, self).search(cr, uid, args, offset=offset, limit=limit, order=order, context=context, count=0)
        for (field,op,value) in args:
            for (id,data) in self.datas.items():
                if op=='=' and data[field]!=value and id in ret: ret.remove(id)
                # The order is important only in Desktop!!  (?)
        if context.get('only_data_changed',False):
             for (id,data) in self.datas.items():
                if not data['changed'] and id in ret:
                    ret.remove(id)
        elif context.get('only_not_linked',False):
             for (id,data) in self.datas.items():
                if data['contact_id_by_xmlid'] and id in ret:
                    ret.remove(id)
        if count: return len(ret)
        return ret


school_saga_student_to_import()

class school_saga_hr_to_import(osv.osv_memory):
    _name = 'school.saga_hr_to_import'
    _max_count = 20000

    def _partners_not_linked(self, cr, uid, ids, field_name, args, context=None):
        query="SELECT rp.id FROM res_partner AS rp LEFT JOIN ir_model_data AS imd ON rp.id=imd.res_id AND imd.model='res.partner' WHERE imd.id IS NULL"
        cr.execute(query)
        return {}.fromkeys(ids,",".join(['0']+[str(x) for (x,) in cr.fetchall()]))

    _columns = {
        'partner_id' : fields.many2one('res.partner','HR',),
        'partners_not_linked' : fields.function(_partners_not_linked,method=True,type='text',),
        'partner_id_by_xmlid' : fields.many2one('res.partner','HR linked',),
        'docent' : fields.boolean('Docent',),
        'importat' : fields.many2one('school.import_saga_data','Import',),
        'xml_id' : fields.char('Id SAGA',size=250,),
        'comment' : fields.text('Comment'),
        'name' : fields.char('Name', size=255,),
        'name_in' : fields.related('partner_id','name',type='char',string='Name In',size=200,),
        'change_name' : fields.boolean('Change Name',),
        'vat': fields.char('VAT',size=15,),
        'vat_in' : fields.related('partner_id','vat',type='char',string='Vat In',size=200,),
        'change_vat' : fields.boolean('Change vat',),
        'changed' : fields.boolean('Changed'),
    }

    def dona_valors_a_changes(self, cr, uid, ids, context=None):
        fields_changing=['change_name','change_vat',]
        for item in self.browse(cr, uid, ids, context=context):
            vals=dict.fromkeys(fields_changing,False)
            if item.partner_id:
                vals['change_name']=(item.partner_id.name!=item.name)
                vals['change_vat']=(item.partner_id.vat!=item.vat)
            vals['changed']=False
            for x in fields_changing:
                vals['changed']=(vals['changed'] or vals[x])
                if vals['changed']: break
            self.write(cr, uid, [item.id], vals)

    def create(self, cr, uid, vals, context=None):
        ret=super(school_saga_hr_to_import, self).create(cr, uid, vals, context=context)
        if ret: self.dona_valors_a_changes(cr, uid, [ret], context)
        return ret

    def write(self, cr, uid, ids, vals, context=None):
        ret=super(school_saga_hr_to_import, self).write(cr, uid, ids, vals, context=context)
        if ret and 'partner_id' in vals: self.dona_valors_a_changes(cr, uid, ids, context)
        return ret

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        if not context: context={}
        ret=super(school_saga_hr_to_import, self).search(cr, uid, args, offset=offset, limit=limit, order=order, context=context, count=0)
        for (field,op,value) in args:
            for (id,data) in self.datas.items():
                if op=='=' and data[field]!=value and id in ret: ret.remove(id)
                # The order is important only in Desktop!!  (?)
        if context.get('only_data_changed',False):
             for (id,data) in self.datas.items():
                if not data['changed'] and id in ret:
                    ret.remove(id)
        elif context.get('only_not_linked',False):
             for (id,data) in self.datas.items():
                if data['partner_id_by_xmlid'] and id in ret:
                    ret.remove(id)
        if count: return len(ret)
        return ret


school_saga_hr_to_import()



class school_saga_assignation_imported(osv.osv_memory):
    _name = 'school.saga_assignation_imported'
    _max_count = 20000

    _columns = {
        'date_from' : fields.datetime('Date from',),
        'date_to' : fields.datetime('Date to',),
        'group_id' : fields.many2one('groups.group',string="Grup existent"),
        'contact_id' : fields.many2one('res.partner.contact','Imported contact',),
        'importat' : fields.many2one('school.import_saga_data',),
        'export_data' : fields.related('importat','export_data',type='date',string='Export data',),
    }

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        ret=super(school_saga_assignation_imported, self).search(cr, uid, args, offset=offset, limit=limit, order=order, context=context, count=0)
        for (field,op,value) in args:
            for (id,data) in self.datas.items():
                if op=='=' and data[field]!=value: ret.remove(id)
        if count: return len(ret)
        return ret

school_saga_assignation_imported()

class school_saga_group_imported(osv.osv_memory):
    _name = 'school.saga_group_imported'

    def name_get(self, cr, uid, ids, context=None):
        res=[]
        for data in self.read(cr, uid, ids, ['name']):
            res.append( (data['id'],data['name']) )
        return res
    
    _columns = {
        'name' : fields.char('Group name',size=40,required=True,),
        'assignation_ids' : fields.one2many('school.saga_assignation_imported','group_imported_id',),
        'importat' : fields.many2one('school.import_saga_data','Import',),
        'group_id' : fields.many2one('groups.group', string='Grup existent',),
    }
    
    def create(self, cr, uid, vals, context=None):
        obj=self.pool.get('school.saga_last_rel_group_imported')
        vals['group_id']=False
        for data in obj.read(cr, uid, obj.search(cr, uid, [('name','=',vals['name'])]), ['group_id',]):
            vals['group_id']=data['group_id'] and data['group_id'][0]
        return super(school_saga_group_imported, self).create(cr, uid, vals, context=context)

    def write(self, cr, uid, ids, vals, context=None):
        for gi in self.browse(cr, uid, ids):
            obj=self.pool.get('school.saga_last_rel_group_imported')
            obj_ids=obj.search(cr, uid, [('name','=',gi.name)])
            if not obj_ids:
                obj_ids.append(obj.create(cr, uid, {'name': gi.name}))
            vals2={}
            for key in ('group_id',):
                if key in vals: vals2[key]=vals[key]
            obj.write(cr, uid, obj_ids, vals2)
        return super(school_saga_group_imported, self).write(cr, uid, ids, vals, context=context)

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        ret=super(school_saga_group_imported, self).search(cr, uid, args, offset=offset, limit=limit, order=order, context=context, count=0)
        for (field,op,value) in args:
            for (id,data) in self.datas.items():
                if op=='=' and data[field]!=value: ret.remove(id)
        if context.get('only_not_linked',False):
             for (id,data) in self.datas.items():
                if data['group_id'] and id in ret:
                    ret.remove(id)
        if count: return len(ret)
        return ret
    
school_saga_group_imported()

class school_saga_classe_to_create(osv.osv_memory):
    _name = 'school.saga_classe_to_create'
    _max_count = 20000

    def name_get(self, cr, uid, ids, context=None):
        res=[]
        for data in self.browse(cr, uid, ids):
            res.append( (data.id,"%s - %s" % (data.group_id.name,data.course_id.name,) ) )
        return res
    
    _columns = {
        'course_id' : fields.many2one('school.course','Course',),
        'group_id' : fields.many2one('groups.group', string='Grup',),
        'teacher_id' : fields.many2one('school.teacher', string='Teacher',),
        'week_hours' : fields.integer('Week hours',required=True,),
        'importat' : fields.many2one('school.import_saga_data','Import',),
    }
    
    def create(self, cr, uid, vals, context=None):
        wizard=self.pool.get('school.import_saga_data').browse(cr, uid, vals['importat'])
        course=self.pool.get('school.course').browse(cr, uid, vals['course_id'])
        vals['week_hours']=course.min_hours*7/(datetime.strptime(wizard.session_date_to,'%Y-%m-%d')-datetime.strptime(wizard.session_date_from,'%Y-%m-%d')).days + 1
        return super(school_saga_classe_to_create, self).create(cr, uid, vals, context=context)
    
school_saga_classe_to_create()

class school_saga_last_rel_group_imported(osv.osv):
    _name = 'school.saga_last_rel_group_imported'

    _columns = {
        'group_id' : fields.many2one('groups.group','Grup existent',ondelete='cascade'),
        'name' : fields.char('Group name',size=40,select=1,),
    }

school_saga_last_rel_group_imported()

def get_group_by_name(pool, cr, uid, name):
    sgi_obj=pool.get('school.saga_last_rel_group_imported')
    g_obj=pool.get('groups.group')
    gi_ids=sgi_obj.search(cr, uid, [('name','=',name)])
    if not gi_ids:
        group_id=g_obj.create(cr, uid, {'name': name,})
        sgi_obj.create(cr, uid, {'group_id': group_id, 'name': name, })
    else:
        group_id=sgi_obj.read(cr, uid, gi_ids, ['group_id'])[0]['group_id']
        if not group_id:
            group_id=g_obj.create(cr, uid, {'name': name,})
            sgi_obj.write(cr, uid, gi_ids, {'group_id': group_id,})
    return type(group_id) in (int,long) and group_id or group_id[0] 

def get_students_set_into_group_in_a_date(pool, cr, uid, group_id, session_id, date):
    p_obj=pool.get('groups.participation')
    ga_obj=pool.get('groups.group_assignation')
    ga_ids=ga_obj.search(cr, uid, [('group_id','=',group_id),('datetime_from','<',date),('datetime_to','>',date)])
    participation_ids=[]
    for x in ga_obj.read(cr, uid, ga_ids, ['participation_id']):
        participation_ids.append(type(x['participation_id']) not in (int,long) and x['participation_id'][0] or x['participation_id'])        
    ys=set()
    for x in p_obj.read(cr, uid, participation_ids, ['participant','session_id']):
        participant=type(x['participant']) not in (int,long) and x['participant'][0] or x['participant']
        if x['session_id'] and session_id:
            session_id2=type(x['session_id']) not in (int,long) and x['session_id'][0] or x['session_id']
            if session_id==session_id2:
                ys.add ( participant )
        elif not (session_id or x['session_id']):
            ys.add ( participant )
    return ys

def add_mark(model,xml_id):
    if model=='res.partner': xml_id='rp-'+xml_id
    if model=='res.partner.contact': xml_id='rpc-'+xml_id
    return xml_id

class school_import_saga_data(osv.osv_memory):
    _name = 'school.import_saga_data'
    _columns = {
        'saga_zip' : fields.binary('Zipped Saga XML', filters='*.zip', help='Zipped Saga XML to import',),
        'function_id' : fields.many2one('contact.annotation.function',string='Guardian function',required=True,),
        'tutor_to_import_ids' : fields.one2many('school.saga_tutor_to_import','importat',string='Data Tutor imported',),
        'student_to_import_ids' : fields.one2many('school.saga_student_to_import', 'importat',string='Data Student imported',),
        'hr_to_import_ids' : fields.one2many('school.saga_hr_to_import', 'importat',string='Data HR imported',),
        'group_imported_ids' : fields.one2many('school.saga_group_imported','importat'),
        'assignation_imported_ids' : fields.one2many('school.saga_assignation_imported','importat',),
        'classe_to_create_ids' : fields.one2many('school.saga_classe_to_create','importat',),
        'export_data' : fields.date('SAGA Export Data', ),
        'school_name' : fields.char('School name',size=100, ),
        'state' : fields.selection([('blank','Blank'),('prepared','Prepared'),('verified','Verified'),('computed','Computed'),('executed','Executed'),('finished','Finished'),], string="State"),
        'groups_found' : fields.integer('Groups found'),
        'students_found' : fields.integer('Students found'),
        'tutors_found' : fields.integer('Tutors found'),
        'hr_found' : fields.integer('HR found'),
        'session_id' : fields.many2one('school.session',string='Session',),
        'session_name' : fields.char('Session',size=200,),
        'session_date_from' :fields.date('Session Date From',),
        'session_date_to' :fields.date('Session Date To',),

        'load_data_passed' : fields.boolean('Load data passed',readonly=True,),
        'link_objects_passed' : fields.boolean('Link objects passed',readonly=True,),
        'change_data_passed' : fields.boolean('Change data passed',readonly=True,),
        'group_assignation_passed' : fields.boolean('Group assignation passed',readonly=True,),
        'classe_creation_passed' : fields.boolean('Classe creation passed',readonly=True,),
    }

    def _get_function_default(self, cr, uid, context=None):
        function_ids=self.pool.get('contact.annotation.function').search(cr, uid, [('code','=','GU')])
        return function_ids and function_ids[0] or False

    _defaults = {
        'state': lambda *a: 'blank',
        'function_id': _get_function_default,
    }


    def compute(self, cr, uid, ids, context=None):
        for imp in self.browse(cr, uid, ids):
            val =base64.decodestring(imp.saga_zip)
            fp = StringIO()
            fp.write(val)
            fdata = zipfile.ZipFile(fp, 'r')
            fname = fdata.namelist()[0]
            root = etree.fromstring(fdata.read(fname))
            classes_to_create=set()
            if root.tag=='saga-centre':
                grups=root.find('grups')
                if grups!=None:
                    alumnes_set={}
                    alumnes_set_into={}
                    for grup in grups:
                        atrs=grup.attrib
                        group_id=get_group_by_name(self.pool, cr, uid, atrs['codi'])
                        continguts=grup.find('continguts')
                        for contingut in continguts:
                            course_id=self.get_id_from_xmlid(cr, uid, 'school.course', add_mark('school.course',contingut.get('contingut')), context=context)
                            partner_id=self.get_id_from_xmlid(cr, uid, 'res.partner', add_mark('res.partner',contingut.get('professor')), context=context)
                            teacher_ids=self.pool.get('school.teacher').search(cr, uid, [('partner_id','=',partner_id)])
                            if not teacher_ids:
                                self.pool.get('school.teacher').create(cr, uid, {'partner_id': partner_id,})
                            if course_id and (course_id,group_id) not in classes_to_create:
                                classes_to_create.add((course_id,group_id))
                                self.pool.get('school.saga_classe_to_create').create(cr, uid, {'course_id' : course_id, 'group_id' : group_id, 'importat' : imp.id, 'teacher_id' : type(teacher_ids) in (int,long) and teacher_ids or teacher_ids and teacher_ids[0] or False, })
                        if group_id not in alumnes_set_into:
                            alumnes_set_into[group_id]=get_students_set_into_group_in_a_date(self.pool, cr, uid, group_id, imp.session_id.id, imp.export_data)
                        if group_id not in alumnes_set: alumnes_set[group_id]=set()                        
                        alumnes_set[group_id].update(map(lambda alumne: self.get_id_from_xmlid(cr, uid, 'res.partner.contact', add_mark('res.partner.contact',alumne.get('id')), context=context), grup.find('alumnes')))
                    for group_id in alumnes_set.keys():
                        for contact_id in alumnes_set[group_id]-alumnes_set_into[group_id]:
                            if contact_id:
                                self.pool.get('school.saga_assignation_imported').create(cr, uid, {'contact_id': contact_id, 'date_from': imp.export_data+' 00:00:00', 'group_id': group_id, 'importat': imp.id})
                            else:
                                pass # Error !!!
                        for contact_id in alumnes_set_into[group_id]-alumnes_set[group_id]:
                            self.pool.get('school.saga_assignation_imported').create(cr, uid, {'contact_id': contact_id, 'date_to': imp.export_data+' 00:00:00', 'group_id': group_id, 'importat': imp.id})
            self.write(cr, uid, [imp.id], {'state': 'computed','change_data_passed': True})

    def link(self, cr, uid, ids, context=None):
        imd_obj=self.pool.get('ir.model.data')
        rp_obj=self.pool.get('res.partner')
        rpc_obj=self.pool.get('res.partner.contact')
        rpa_obj=self.pool.get('res.partner.address')
        stti_obj=self.pool.get('school.saga_tutor_to_import')
        ssti_obj=self.pool.get('school.saga_student_to_import')
        shti_obj=self.pool.get('school.saga_hr_to_import')
        cpf_obj=self.pool.get('contact.partner.function')
        """
        Presentar llista d'alumnes en el SAGA que no apareixen com a contactes buscant per l'identificador.
        Donar l'opció d'enllaçar amb contactes (amb assignacions de grup classe que estiguin enllaçats amb els grups interns d'aquesta importació) que no estiguin ja enllaçats amb aquest .
        """
        for imp in self.browse(cr, uid, ids):
            for ti in imp.tutor_to_import_ids:
                tutor_id=ti.tutor_id.id
                if tutor_id:
                    tutor_ids_existing=imd_obj.search(cr, uid, [('model','=','res.partner'),('res_id','=',tutor_id)])
                    imd_obj.unlink(cr, uid, tutor_ids_existing)
                    tutor_ids_existing=imd_obj.search(cr, uid, [('model','=','res.partner'),('name','=',ti.xml_id)])
                    imd_obj.unlink(cr, uid, tutor_ids_existing)
                    imd_obj.create(cr, uid, {'model': 'res.partner', 'name': ti.xml_id, 'res_id': tutor_id})
                else:
                    tutor_id=rp_obj.create(cr, uid, {'name':ti.name,'vat':ti.vat,'doc_type':ti.doc_type}, context=context)
                    rpa_obj.create(cr, uid, {'partner_id': tutor_id, 'street': ti.street, 'city': ti.city, 'zip': ti.zip})
                    tutor_ids_existing=imd_obj.search(cr, uid, [('model','=','res.partner'),('res_id','=',tutor_id)])
                    imd_obj.unlink(cr, uid, tutor_ids_existing)
                    tutor_ids_existing=imd_obj.search(cr, uid, [('model','=','res.partner'),('name','=',ti.xml_id)])
                    imd_obj.unlink(cr, uid, tutor_ids_existing)
                    imd_obj.create(cr, uid, {'model': 'res.partner', 'name': ti.xml_id, 'res_id': tutor_id})
                    stti_obj.write(cr, uid, [ti.id], {'tutor_id': tutor_id})

            for ti in imp.hr_to_import_ids:
                partner_id=ti.partner_id.id
                if partner_id:
                    partner_ids_existing=imd_obj.search(cr, uid, [('model','=','res.partner'),('res_id','=',tutor_id)])
                    imd_obj.unlink(cr, uid, partner_ids_existing)
                    partner_ids_existing=imd_obj.search(cr, uid, [('model','=','res.partner'),('name','=',ti.xml_id)])
                    imd_obj.unlink(cr, uid, partner_ids_existing)
                    imd_obj.create(cr, uid, {'model': 'res.partner', 'name': ti.xml_id, 'res_id': partner_id})
                else:
                    partner_id=rp_obj.create(cr, uid, {'name':ti.name,'vat':ti.vat,}, context=context)
                    imd_obj.create(cr, uid, {'model': 'res.partner', 'name': ti.xml_id, 'res_id': partner_id})
                    shti_obj.write(cr, uid, [ti.id], {'partner_id': partner_id})
                    if ti.docent:
                        self.pool.get('school.teacher').create(cr, uid, {'partner_id': partner_id,})

            for ti in imp.student_to_import_ids:
                contact_id=ti.contact_id.id
                if contact_id:
                    contact_ids_existing=imd_obj.search(cr, uid, [('model','=','res.partner.contact'),('res_id','=',contact_id)])
                    imd_obj.unlink(cr, uid, contact_ids_existing)
                    contact_ids_existing=imd_obj.search(cr, uid, [('model','=','res.partner.contact'),('name','=',ti.xml_id)])
                    imd_obj.unlink(cr, uid, contact_ids_existing)
                    imd_obj.create(cr, uid, {'model': 'res.partner.contact', 'name': ti.xml_id, 'res_id': contact_id})
                else:
                    contact_id=rpc_obj.create(cr, uid, {'name':ti.name, 'first_name':ti.first_name, 'birthdate':ti.birthdate, 'vat':ti.vat, 'doc_type':ti.doc_type, 'gender':ti.gender}, context=context)
                    contact_ids_existing=imd_obj.search(cr, uid, [('model','=','res.partner.contact'),('res_id','=',contact_id)])
                    imd_obj.unlink(cr, uid, contact_ids_existing)
                    contact_ids_existing=imd_obj.search(cr, uid, [('model','=','res.partner.contact'),('name','=',ti.xml_id)])
                    imd_obj.unlink(cr, uid, contact_ids_existing)
                    imd_obj.create(cr, uid, {'model': 'res.partner.contact', 'name': ti.xml_id, 'res_id': contact_id})
                    ssti_obj.write(cr, uid, [ti.id], {'contact_id': contact_id})
                    
                    
                # Esborra tutors legals actuals ...
                cpf_ids=cpf_obj.search(cr, uid, [('contact_id','=',contact_id),('function_id','=',imp.function_id.id)])
                cpf_obj.unlink(cr, uid, cpf_ids)
                # ... i recrea'ls amb la info del SAGA
                if ti.xml_id_tutor1:
                    tutor_id=self.get_id_from_xmlid(cr, uid, 'res.partner', ti.xml_id_tutor1)
                    if tutor_id: cpf_obj.create(cr, uid, {'contact_id': contact_id, 'function_id': imp.function_id.id, 'partner_id': tutor_id})
                if ti.xml_id_tutor2:
                    tutor_id=self.get_id_from_xmlid(cr, uid, 'res.partner', ti.xml_id_tutor2)
                    if tutor_id: cpf_obj.create(cr, uid, {'contact_id': contact_id, 'function_id': imp.function_id.id, 'partner_id': tutor_id})
            stti_obj.dona_valors_a_changes(cr, uid, [x.id for x in imp.tutor_to_import_ids], context)
            ssti_obj.dona_valors_a_changes(cr, uid, [x.id for x in imp.student_to_import_ids], context)
        self.write(cr, uid, ids, {'state': 'verified', 'link_objects_passed': True})
            
        return {
                'name': _('Import SAGA Data: To Change Data'),
                'view_type': 'form',
                'view_mode': 'form',
                'res_model': 'school.import_saga_data',
                'res_id' : ids[0],
                'views' : [(self.get_id_from_xmlid(cr, uid, 'ir.ui.view', 'school_import_saga_change_data_form_view_2', context=context),'form')],
                'view_id': False,
                'target' : 'new',
                'type': 'ir.actions.act_window',
                'context' : {'only_data_changed': True,},
        }


    def change_data(self, cr, uid, ids, context=None):
        rp_obj=self.pool.get('res.partner')
        rpa_obj=self.pool.get('res.partner.address')
        rpc_obj=self.pool.get('res.partner.contact')
        for imp in self.browse(cr, uid, ids):
            for ti in imp.tutor_to_import_ids:
                vals={};vals2={}
                if ti.change_name: vals['name']=ti.name
                if ti.change_vat: vals['vat']=ti.vat
                if ti.change_doc_type: vals['doc_type']=ti.doc_type
                if ti.change_street: vals2['street']=ti.street
                if ti.change_city: vals2['city']=ti.city
                if ti.change_zip: vals2['zip']=ti.zip
                if vals: rp_obj.write(cr, uid, [ti.tutor_id.id], vals, context=context)
                if vals2: rpa_obj.write(cr, uid, rpa_obj.search(cr, uid, [('partner_id','=',ti.tutor_id.id)]), vals2, context=context)
            for ti in imp.hr_to_import_ids:
                vals={};
                if ti.change_name: vals['name']=ti.name
                if ti.change_vat: vals['vat']=ti.vat
                if vals: rp_obj.write(cr, uid, [ti.partner_id.id], vals, context=context)
            for ti in imp.student_to_import_ids:
                vals={};
                if ti.change_name: vals['name']=ti.name
                if ti.change_vat: vals['vat']=ti.vat
                if ti.change_doc_type: vals['doc_type']=ti.doc_type
                if ti.change_first_name: vals['first_name']=ti.first_name
                if ti.change_birthdate: vals['birthdate']=ti.birthdate
                if ti.change_gender: vals['gender']=ti.gender
                if vals: rpc_obj.write(cr, uid, [ti.contact_id.id], vals, context=context)
        self.compute(cr, uid, ids, context=context)
        return {
                'name': _('Import SAGA Data: Group Assignations to Modify'),
                'view_type': 'form',
                'view_mode': 'form',
                'res_model': 'school.import_saga_data',
                'res_id' : ids[0],
                'views' : [(self.get_id_from_xmlid(cr, uid, 'ir.ui.view', 'school_import_saga_assignations_form_view_3', context=context),'form')],
                'view_id': False,
                'target' : 'new',
                'type': 'ir.actions.act_window',
        }
    
    def get_id_from_xmlid(self, cr, uid, model, xml_id, context=None):
        imd_obj=self.pool.get('ir.model.data')
        id=False
        ir_model_data_ids=imd_obj.search(cr, uid, [('model','=',model),('name','=',xml_id)])
        if ir_model_data_ids:
            for data in imd_obj.read(cr, uid, ir_model_data_ids, ['res_id']):
                if not self.pool.get(model).search(cr, uid, [('id','=',data['res_id'])]):
                    imd_obj.unlink(cr, uid, [data['id']])
                else:
                    id=data['res_id']
        return id
        
    def load(self, cr, uid, ids, context=None):
        sgi_obj=self.pool.get('school.saga_group_imported')
        for imp in self.browse(cr, uid, ids):
            vals={}
            val =base64.decodestring(imp.saga_zip)
            fp = StringIO()
            fp.write(val)
            fdata = zipfile.ZipFile(fp, 'r')
            fname = fdata.namelist()[0]
            root = etree.fromstring(fdata.read(fname))
            if root.tag=='saga-centre':
                vals['export_data']=(datetime.strptime(root.get('data-exportacio'),'%d/%m/%Y')).strftime('%Y-%m-%d')
                centre=root.find('centre')
                if centre!=None:
                    vals['school_name']=centre.get('nom')
                    vals['session_name']=centre.get('curs')
                    session_ids=self.pool.get('school.session').search(cr, uid, [('name','=',vals['session_name'])])
                    vals['session_id']=session_ids and session_ids[0] or False
                    if not vals['session_id'] and not (imp.session_date_from and imp.session_date_to):
                        self.write(cr, uid, [imp.id], vals, context=context)
                        return {
                                'name': _('Import SAGA Data: Session Data'),
                                'view_type': 'form',
                                'view_mode': 'form',
                                'res_model': 'school.import_saga_data',
                                'res_id' : imp.id,
                                'views' : [(self.get_id_from_xmlid(cr, uid, 'ir.ui.view', 'school_import_saga_load_data_form_view_0b', context=context),'form')],
                                'view_id': False,
                                'target' : 'new',
                                'type': 'ir.actions.act_window',
                        }
                    if not vals['session_id']:
                        vals['session_id']=self.pool.get('school.session').create(cr, uid, {'name': vals['session_name'], 'date_from': imp.session_date_from, 'date_to': imp.session_date_to})
                    else:
                        fields=['date_from','date_to']
                        dicci=self.pool.get('school.session').read(cr, uid, [vals['session_id']],fields, context)[0]
                        for field in fields: vals['session_'+field]=dicci[field]
                personal=root.find('personal')
                if personal!=None:
#                    vals['personal_found']=len(personal)
                    for persona in personal:
                        atrs=persona.attrib
                        xml_id=add_mark('res.partner',atrs.get('id'))
                        persona_id=self.get_id_from_xmlid(cr, uid, 'res.partner', xml_id, context=context)
                        dicci={'name': atrs.get('cognom1','')+(('cognom2' in atrs) and (' '+atrs['cognom2']) or '')+(('nom' in atrs) and (', '+atrs['nom']) or ''),
                            'vat': atrs.get('documentidentitat',False),'docent' : atrs.get('tipus',False)=='DOCENT',
                            'importat' : imp.id, 'comment': 'not found', 'partner_id': persona_id, 'partner_id_by_xmlid': persona_id, 'xml_id': xml_id}
                        if dicci['partner_id']: dicci['comment']='found'
                        self.pool.get('school.saga_hr_to_import').create(cr, uid, dicci)

                plans_estudis=root.findall('plans-estudi/pla-estudis')
                if plans_estudis!=None:
                    for pla_estudis in plans_estudis:
                        continguts=pla_estudis.findall('contingut')
                        if continguts!=None:
                            for contingut in continguts:
                                atrs=contingut.attrib
                                xml_id=add_mark('school.course',atrs.get('id'))
                                course_id=self.get_id_from_xmlid(cr, uid, 'school.course', xml_id, context=context)
                                if not course_id:
                                    course_id=self.pool.get('school.course').create(cr, uid, {'name': atrs.get('nom'), 'code': atrs.get('codi')})
                                    self.pool.get('ir.model.data').create(cr, uid, {'model': 'school.course', 'name': xml_id, 'res_id': course_id})
                                            
                tutors_legals=root.find('tutors-legals')
                if tutors_legals!=None:
                    vals['tutors_found']=len(tutors_legals)
                    for tutor_legal in tutors_legals:
                        atrs=tutor_legal.attrib
                        xml_id=add_mark('res.partner',atrs.get('id'))
                        tutor_id=self.get_id_from_xmlid(cr, uid, 'res.partner', xml_id, context=context)
                        dicci={'name': atrs.get('cognom1','')+(('cognom2' in atrs) and (' '+atrs['cognom2']) or '')+(('nom' in atrs) and (', '+atrs['nom']) or ''),
                            'vat': atrs.get('documentidentitat',False), 'doc_type': atrs.get('tipusdocument',False),'street': atrs.get('adreca',False) ,
                            'city': atrs.get('nomlocalitat',False), 'zip': atrs.get('codipostal',False),
                            'importat' : imp.id, 'comment': 'not found', 'tutor_id': tutor_id, 'tutor_id_by_xmlid': tutor_id, 'xml_id': xml_id}
                        if dicci['tutor_id']: dicci['comment']='found'
                        self.pool.get('school.saga_tutor_to_import').create(cr, uid, dicci)

                students=root.find('alumnes')
                if students!=None:
                    vals['students_found']=len(students)
                    for student in students:
                        atrs=student.attrib
                        xml_id=add_mark('res.partner.contact',student.get('id'))
                        contact_id=self.get_id_from_xmlid(cr, uid, 'res.partner.contact', xml_id, context=context)
                        dicci={'name': atrs.get('cognom1','')+(('cognom2' in atrs) and (' '+atrs['cognom2']) or ''),
                            'first_name': atrs.get('nom',''),
                            'birthdate': 'datanaixement' in atrs and datetime.strptime(atrs['datanaixement'],'%d/%m/%Y').strftime('%Y-%m-%d') or False,
                            'vat': atrs.get('documentidentitat',False), 'doc_type': atrs.get('tipusdocument',False), 'gender': atrs.get('sexe',False)=='H' and 'M' or 'W',
                            'importat' : imp.id, 'comment': 'not found', 'contact_id': contact_id, 'contact_id_by_xmlid': contact_id, 'xml_id': xml_id, 'xml_id_tutor1': atrs.get('tutor1',False), 'xml_id_tutor2': atrs.get('tutor2',False),}
                        if dicci['contact_id']: dicci['comment']=_('found')
                        self.pool.get('school.saga_student_to_import').create(cr, uid, dicci)
                            
                grups=root.find('grups')
                if grups!=None:
                    vals['groups_found']=len(grups)
                    for grup in grups:
                        atrs=grup.attrib
                        imported_group_id=0
                        for (ig_id,ig) in sgi_obj.datas.items():
                            if ig['name']==atrs['codi'] and ig['importat']==imp.id:
                                imported_group_id=ig_id
                        if not imported_group_id:
                            imported_group_id=sgi_obj.create(cr, uid, {'name': atrs['codi'], 'importat': imp.id,})
            vals['state']='prepared'
            vals['load_data_passed']=True
            self.write(cr, uid, [imp.id], vals)
            
        return {
                'name': _('Import SAGA Data: Objects to Link'),
                'view_type': 'form',
                'view_mode': 'form',
                'res_model': 'school.import_saga_data',
                'res_id' : ids[0],
                'views' : [(self.get_id_from_xmlid(cr, uid, 'ir.ui.view', 'school_import_saga_link_objects_form_view_1', context=context),'form')],
                'view_id': False,
                'target' : 'new',
                'type': 'ir.actions.act_window',
                'context' : {'only_not_linked': True,},
        }

    def execute(self, cr, uid, ids, context=None):
        gp_obj=self.pool.get('groups.participation')
        ga_obj=self.pool.get('groups.group_assignation')
        for imp in self.browse(cr, uid, ids):
            for ai in imp.assignation_imported_ids:
                participation_ids=gp_obj.search(cr, uid, [('participant','=',ai.contact_id.id),('session_id','=',imp.session_id.id)])
                if not participation_ids:
                    participation_ids=[gp_obj.create(cr, uid, {'participant': ai.contact_id.id, 'session_id': imp.session_id.id})]
                if ai.date_from and participation_ids:
                    for part_id in participation_ids:
                        ga_obj.create(cr, uid, {'participation_id': part_id, 'group_id': ai.group_id.id, 'datetime_from': ai.date_from})
                if ai.date_to:
                    ids2=ga_obj.search(cr, uid, [('participation_id','in',participation_ids),('group_id','=',ai.group_id.id),('datetime_from','<',imp.export_data+' 00:00:00'),('datetime_to','>',imp.export_data+' 00:00:00')])
                    ga_obj.write(cr, uid, ids2, {'datetime_to': ai.date_to})
        self.write(cr, uid, ids, {'state': 'executed','group_assignation_passed': True})
        return {
                'name': _('Import SAGA Data: Classe Creation'),
                'view_type': 'form',
                'view_mode': 'form',
                'res_model': 'school.import_saga_data',
                'res_id' : ids[0],
                'views' : [(self.get_id_from_xmlid(cr, uid, 'ir.ui.view', 'school_import_saga_classes_form_view_4', context=context),'form')],
                'view_id': False,
                'target' : 'new',
                'type': 'ir.actions.act_window',
        }

    def classe_creation(self, cr, uid, ids, context=None):
        cl_obj=self.pool.get('school.classe')
        iwl_obj=self.pool.get('school.impartition_week_line')
        td_obj=self.pool.get('school.teacher_data')
        for imp in self.browse(cr, uid, ids):
            for classe_data in imp.classe_to_create_ids:
                classe_id=cl_obj.create(cr, uid, {'group_id': classe_data.group_id.id, 'course_id': classe_data.course_id.id, 'date_from': imp.session_date_from, 'date_to': imp.session_date_to, })
                for counter in range(classe_data.week_hours):
                    iwl_id=iwl_obj.create(cr, uid, {'classe_id' : classe_id, 'duration' : 1,})
                    if classe_data.teacher_id:
                        td_obj.create(cr, uid, {'iwl_id': iwl_id, 'teacher_id' : classe_data.teacher_id.id,})
        self.write(cr, uid, ids, {'state': 'finished','classe_creation_passed': True})
        return {
                'name': _('Import SAGA Data: End Wizard'),
                'view_type': 'form',
                'view_mode': 'form',
                'res_model': 'school.import_saga_data',
                'res_id' : ids[0],
                'views' : [(self.get_id_from_xmlid(cr, uid, 'ir.ui.view', 'school_import_saga_form_view_end', context=context),'form')],
                'view_id': False,
                'target' : 'new',
                'type': 'ir.actions.act_window',
        }


    
school_import_saga_data()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

