# -*- coding: utf-8 -*-
##############################################################################
#
#    school module for OpenERP
#    Copyright (C) 2010 Tecnoba S.L. (http://www.tecnoba.com)
#       Pere Ramon Erro Mas <pereerro@tecnoba.com> All Rights Reserved.
#
#    This file is a part of school module
#
#    school OpenERP module is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    school OpenERP module is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv, fields
from datetime import datetime

class res_partner(osv.osv):
    _name = 'res.partner'
    _inherit = 'res.partner'
    
    _columns = {
        'school_presence' : fields.boolean('School Presence',select=1,),
        'school_note' : fields.text('Note'),
        'school_note_datetime_to' : fields.datetime('To time'),
        'school_note_counter' : fields.integer('Counter',help="Note valid n times decrementing each visualization. Zero is equal to not visualization. Negative number means always.")
    }
    
    def get_school_note(self, cr, uid, ids, context=None):
        ret=[]
        ids2=[]
        for partner in self.browse(cr, uid, ids):
            if (not partner.school_note_datetime_to or datetime.today().strftime('%Y-%m-%d %H:%M:%S')<partner.school_note_datetime_to) and partner.school_note_counter!=0:
                ret.append( (partner.id,partner.school_note) )
                ids2.append(partner.id)
            else:
                ret.append( (partner.id,"") )
        if ids2:
            cr.execute("UPDATE res_partner SET school_note_counter=school_note_counter-1 WHERE id IN %s",(tuple(ids2),))

        return ret
res_partner()

class school_teacher_access(osv.osv):
    _name = 'school.teacher_access'


    _columns = {
        'partner_id' : fields.many2one('res.partner','Partner', select=1,ondelete='cascade',help="The partner",required=True,),
        'datetime': fields.datetime('Time',required=True,),
    }

    _defaults = {
        'datetime' : lambda *a: datetime.today().strftime('%Y-%m-%d %H:%M:%S'),
    }
    
    def create(self, cr, uid, vals, context=None):
        obj=self.pool.get('res.partner')
        obj.write(cr, uid, [vals['partner_id']], {'school_presence': not obj.read(cr, uid, [vals['partner_id']], ['school_presence'])[0]['school_presence'],})
        return super(school_teacher_access, self).create(cr, uid, vals, context=context)
        
school_teacher_access()

class school_teacher_changes(osv.osv):
    _name = 'school.teacher_changes'
    
    _columns = {
        'teacher_id_out' : fields.many2one('school.teacher','Teacher out',require=True,readonly=True,),
        'teacher_id_in' : fields.many2one('school.teacher','Teacher in',require=True,readonly=True,),
        'seance_id_from' : fields.many2one('school.seance','Seance from',require=True,readonly=True,),
        'seance_id_to' : fields.many2one('school.seance','Seance to',require=True,readonly=True,),
    }
school_teacher_changes()

class school_course(osv.osv):
    _name = 'school.course'
    _inherit = 'school.course'

    _columns = {
        'order_for_change_teacher' : fields.integer('Order FST',help='Order for teacher changes',),
    }
school_course()

class school_teacher_seance(osv.osv):
    _name = 'school.teacher_seance'
    _inherit = 'school.teacher_seance'
    
    _columns = {
        'no_problem' : fields.boolean('No problem',),
    }
    
    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        context2=context and dict(context) or {}
        in_context_to_process=context2.pop('search_order_for_change_teacher',False)
        in_context_to_process2=context2.pop('search_seances_for_change_teacher',False)
        if not count and in_context_to_process:
            # No filtrem, nomes volem que estigui en un ordre determinat
            ret=super(school_teacher_seance, self).search(cr, uid, args, offset=0, limit=None, order=None, context=context2, count=False)
            if ret:
                query="SELECT ts.id FROM school_teacher_seance AS ts INNER JOIN school_seance AS s ON ts.seance_id=s.id INNER JOIN school_classe AS ca ON s.classe_id=ca.id INNER JOIN school_course AS co ON ca.course_id=co.id WHERE ts.id IN %s ORDER BY co.order_for_change_teacher OFFSET %s"
                params=[tuple(ret),offset]
                if limit:
                    query+=" LIMIT %s"
                    params+=[limit]
                cr.execute(query,params)
                ret=[x for (x,) in cr.fetchall()]
        else:
            if in_context_to_process2:
                # Filtrem perque nomes surtin les assignacions de professor a sessio en les que el professor no estigui al centre. Tampoc presentem les quines haguem marcat com a 'no problem'
                query="""
                SELECT ts.id
                FROM school_seance AS s
                    INNER JOIN school_teacher_seance AS ts ON ts.seance_id=s.id AND NOT COALESCE(ts.no_problem,false)
                    INNER JOIN school_teacher AS t ON t.id=ts.teacher_id
                    INNER JOIN res_partner AS p ON p.id=t.partner_id AND NOT p.school_presence
                    WHERE s.group_id IS NOT NULL AND date_trunc('day',s.date)=date_trunc('day',now()) AND s.state NOT IN ('suspended')
                """
                cr.execute(query)
                args+=[('id','in',[x for (x,) in cr.fetchall()])]
            ret=super(school_teacher_seance, self).search(cr, uid, args, offset=offset, limit=limit, order=order, context=context2, count=count)
        return ret

    def read(self, cr, uid, ids, fields, context=None, load='_classic_write'):
        context2=context and dict(context) or {}
        in_context_to_process=context2.pop('search_order_for_change_teacher',False)
        ret=super(school_teacher_seance, self).read(cr, uid, ids, fields, context=context2, load=load)
        if in_context_to_process:
            query="SELECT ts.id,0-COALESCE(co.order_for_change_teacher,0) FROM school_teacher_seance AS ts INNER JOIN school_seance AS s ON ts.seance_id=s.id INNER JOIN school_classe AS ca ON s.classe_id=ca.id INNER JOIN school_course AS co ON ca.course_id=co.id WHERE ts.id IN %s"
            params=[tuple([x['id'] for x in ret]),]
            cr.execute(query,params)
            key_order=dict(cr.fetchall())
            ret=sorted(ret, key= lambda x: key_order[x['id']] )
        return ret


school_teacher_seance()
        
class school_change_teachers_wizard(osv.osv_memory):
    _name = 'school.change_teachers_wizard'

    _columns = {
        'teacher_seance_ids' : fields.many2many('school.teacher_seance','dummy_rel','id1','id2',string='Seances',readonly=True,),
        'teacher_to_change_id' : fields.many2one('school.teacher_seance',string='Teacher to change',context={'search_order_for_change_teacher': True,},),
        'teacher_seances_to_change_ids' : fields.text('domain'),
    }

    def _get_default_teacher_to_change(self, cr, uid, context=None):
        if not context or 'active_ids' not in context or len(context['active_ids'])!=1: return '0'
        query="""
        SELECT DISTINCT ts2.id
        FROM school_teacher_seance AS ts
            INNER JOIN school_seance AS s ON ts.seance_id=s.id
            INNER JOIN school_seance AS s2 ON s2.group_id IS NULL AND s.date>=s2.date AND s.date_to<=s2.date_to
            INNER JOIN school_teacher_seance AS ts2 ON s2.id=ts2.seance_id
            INNER JOIN school_teacher AS t2 ON t2.id=ts2.teacher_id
            INNER JOIN res_partner AS p ON t2.partner_id=p.id AND p.school_presence
            -- Cal saber si ja tenim el professor present substituint alguna altra sessio
            LEFT JOIN school_seance AS s3 ON s3.group_id IS NOT NULL
						AND s3.date<s.date_to
						AND s3.date_to> s.date
						AND s3.state NOT IN ('suspended')
            LEFT JOIN school_teacher_seance AS ts3 ON s3.id=ts3.seance_id AND ts3.teacher_id=ts2.teacher_id
        WHERE ts.id=%s AND ts3.id IS NULL        """
        cr.execute(query,(context['active_ids'][0],))
        return ','.join(['0']+[str(x) for (x,) in cr.fetchall()])
    
    _defaults = {
        'teacher_seance_ids' : lambda self,cr,uid,context: context.get('active_ids',False),
        'teacher_seances_to_change_ids' : _get_default_teacher_to_change,
    }
    
    def change_teacher(self, cr, uid, ids, context=None):
        for item in self.browse(cr, uid, ids):
            if item.teacher_to_change_id:
                self.pool.get('school.teacher_seance').write(cr, uid, [item.teacher_seance_ids[0].id], {'teacher_id': item.teacher_to_change_id.teacher_id.id,})
                self.pool.get('school.teacher_changes').create(cr, uid, {'teacher_id_out': item.teacher_seance_ids[0].teacher_id.id, 'teacher_id_in': item.teacher_to_change_id.teacher_id.id, 'seance_id_from': item.teacher_to_change_id.seance_id.id, 'seance_id_to': item.teacher_seance_ids[0].seance_id.id})
        return {'type': 'ir.actions.act_window_close'}

    def no_problem(self, cr, uid, ids, context=None):
        for item in self.browse(cr, uid, ids):
            self.pool.get('school.teacher_seance').write(cr, uid, [x.id for x in item.teacher_seance_ids], {'no_problem': True,})
        return {'type': 'ir.actions.act_window_close'}

    def act_cancel(self, cr, uid, ids, context=None):
        return {'type': 'ir.actions.act_window_close'}


school_change_teachers_wizard()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
